import importlib
import re


def parse_uri(uri):
    pattern = re.compile(
        r'^(?P<vendor>\w+)(?:\+(?P<scheme>http|https))?://(?P<conn_str>.*)',
        re.X,
    )

    m = pattern.match(uri)

    if m is not None:
        components = m.groupdict()

        return ServiceConnection(components['vendor'], components['scheme'], components['conn_str'])

    return None


def create_service(uri):
    sc = parse_uri(uri)

    if sc is not None:
        return Service.create(sc)

    return None


class ServiceConnection:
    def __init__(self, vendor, scheme, conn_str):
        self.vendor = vendor
        self.scheme = scheme
        self.conn_str = conn_str


class Service:
    @staticmethod
    def create(sc):
        module = importlib.import_module('apialchemy.vendors.' + sc.vendor)

        return module.Service(sc.scheme, sc.conn_str)



import os
import re
import urllib3

from urllib import parse

from .. import BaseService

from .client import ExtraHopClient


class Service(BaseService):
    _conn_params = {}

    application = None

    def __init__(self, scheme, conn_str):
        super(Service, self).__init__(scheme, conn_str)

        pattern = re.compile(
            r"""
                (?P<apikey>[^:/]*)
                @(?:
                    \[(?P<ipv6host>[^/]+)\] |
                    (?P<ipv4host>[^/:]+)
                )
                (?::(?P<port>[\d]+))?
            """,
            re.X
        )

        m = pattern.match(self._conn_str)

        if m is not None:
            components = m.groupdict()

            components['apikey'] = parse.unquote(components['apikey'])

            if self._scheme is not None:
                components['scheme'] = self._scheme

            ipv4host = components.pop('ipv4host')
            ipv6host = components.pop('ipv6host')

            components['host'] = ipv4host or ipv6host

            self._conn_params = components

    @property
    def client(self):
        verify = os.getenv('APIALCHEMY_EXTRAHOP_SSL_VERIFY', 'true').lower() == 'true'

        if not verify:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        return ExtraHopClient(**self._conn_params, verify=verify)



import os
import re
import urllib3

from time import sleep

from urllib import parse

from .. import BaseService

from splunklib.client import connect
from splunklib import results


class Service(BaseService):
    _conn_params = {}

    application = None

    def __init__(self, scheme, conn_str):
        super(Service, self).__init__(scheme, conn_str)

        pattern = re.compile(
            r"""
                (?P<username>[^:/]*)
                :(?P<password>.*)
                @(?:
                    \[(?P<ipv6host>[^/]+)\] |
                    (?P<ipv4host>[^/:]+)
                )
                (?::(?P<port>[\d]+))?
                (?:/(?P<application>.*))?
            """,
            re.X
        )

        m = pattern.match(self._conn_str)

        if m is not None:
            components = m.groupdict()

            components['username'] = parse.unquote(components['username'])
            components['password'] = parse.unquote(components['password'])

            if self._scheme is not None:
                components['scheme'] = self._scheme

            ipv4host = components.pop('ipv4host')
            ipv6host = components.pop('ipv6host')

            components['host'] = ipv4host or ipv6host

            port = components.pop('port')

            if port is not None:
                components['port'] = port

            application = components.pop('application')

            if application is not None:
                self.application = parse.unquote(application)

            self._conn_params = components

    @property
    def client(self):
        verify = os.getenv('APIALCHEMY_SPLUNK_SSL_VERIFY', 'true').lower() == 'true'

        if not verify:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        return connect(**self._conn_params, verify=verify)

    @staticmethod
    def search(client, query, **kwargs):
        if 'exec_mode' not in kwargs.keys():
            kwargs['exec_mode'] = 'blocking'

        job = client.search(query, **kwargs)

        if kwargs['exec_mode'] == 'normal':
            while True:
                while not job.is_ready():
                    pass

                if job['isDone'] == '1':
                    break

                sleep(2)

        result_count = int(job['resultCount'])
        offset = 0
        count = int(client.confs['limits']['restapi']['maxresultrows'])

        reader = []

        while offset < result_count:
            kwargs_paginate = {
                'count': count,
                'offset': offset
            }

            reader += results.ResultsReader(job.results(**kwargs_paginate))

            offset += count

        job.cancel()

        return reader



from abc import ABCMeta, abstractmethod


class BaseService:
    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self, scheme, conn_str):
        self._scheme = scheme
        self._conn_str = conn_str



import os
import re

from setuptools import find_packages, setup

with open(
    os.path.join(os.path.dirname(__file__), 'src', 'apialchemy', '__init__.py')
) as v_file:
    VERSION = (
        re.compile(r""".*__version__ = ["']([^\n]*)['"]""", re.S)
        .match(v_file.read())
        .group(1)
    )

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as r_file:
    readme = r_file.read()

setup(name='APIAlchemy',
      version=VERSION,
      description='API toolkit for Python, modeled after SQLAlchemy',
      long_description=readme,
      long_description_content_type='text/markdown',
      license='MIT',
      url='https://github.com/homedepot/apialchemy',
      author='Mike Phillipson',
      author_email='MICHAEL_PHILLIPSON1@homedepot.com',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      install_requires=[
          'AppDynamicsRESTx',
          'orionsdk',
          'prometheus_client',
          'prometheus-api-client',
          'PyGithub',
          'splunk-sdk'
      ],
      zip_safe=False)



import base64
import os
import re
import urllib3

from urllib import parse

from .. import BaseService

from prometheus_api_client import PrometheusConnect, Metric, MetricsList


class Service(BaseService):
    _conn_params = {}

    def __init__(self, scheme, conn_str):
        super(Service, self).__init__(scheme, conn_str)

        pattern = re.compile(
            r"""
                (?:
                    (?P<username>[^:/]*)
                    :(?P<password>.*)@
                )?
                (?:
                    \[(?P<ipv6host>[^/]+)\] |
                    (?P<ipv4host>[^/:]+)
                )
                (?::(?P<port>[\d]+))?
            """,
            re.X
        )

        m = pattern.match(self._conn_str)

        if m is not None:
            components = m.groupdict()

            username = components.pop('username')
            password = components.pop('password')

            if username is not None and password is not None:
                encoding = os.getenv('APIALCHEMY_PROMETHEUS_AUTH_ENCODING', 'utf-8')

                auth_str = '%s:%s' % (parse.unquote(username), parse.unquote(password))
                auth_bytes = auth_str.encode(encoding)
                base64_bytes = base64.b64encode(auth_bytes)

                components['headers'] = {
                    'Authorization': base64_bytes.decode(encoding)
                }

            ipv4host = components.pop('ipv4host')
            ipv6host = components.pop('ipv6host')

            host = ipv4host or ipv6host

            if self._scheme is not None:
                scheme = self._scheme
            else:
                scheme = 'http'

            components['url'] = scheme + '://' + host

            port = components.pop('port')

            if port is not None:
                components['url'] += ':' + port

            self._conn_params = components

    @property
    def client(self):
        verify = os.getenv('APIALCHEMY_PROMETHEUS_SSL_VERIFY', 'true').lower() == 'true'

        if not verify:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        return PrometheusConnect(**self._conn_params, disable_ssl=not verify)

    @staticmethod
    def get_metric_obj(metric_data):
        return Metric(metric_data)

    @staticmethod
    def get_metric_obj_list(metric_data):
        return MetricsList(metric_data)
